#!/bin/bash

minirc_boot_log()
{
	cur_time=$(date +%Y-%m-%d,%H:%m:%s)
	if [ ! -e /var/davinci/log/boot.log ]; then
		mkdir -p /var/davinci/log
		echo "$cur_time  create log file" > /var/davinci/log/boot.log
	else
		#file is too large, remove it
		fisize=$(stat -c %s /var/davinci/log/boot.log)
		if [ $((fisize)) -gt 5242880 ]; then
			rm /var/davinci/log/boot.log
			echo "$cur_time  create log file" > /var/davinci/log/boot.log
		fi
	fi

	echo "$cur_time  $1" >> /var/davinci/log/boot.log
}

minirc_upgrade_log()
{
    cur_time=$(date +%Y-%m-%d,%H:%m:%s)
	if [ ! -e /var/davinci/log/upgrade.log ]; then
		mkdir -p /var/davinci/log
		echo "$cur_time  create log file" > /var/davinci/log/upgrade.log
	fi
    echo "$cur_time  $1" >> /var/davinci/log/upgrade.log
}

#mini start to init
system_init()
{
	if [ -e /var/davinci/scripts/minirc_sys_init.sh ]; then
		minirc_boot_log "system start to init"
		chmod 550 /var/davinci/scripts/minirc_sys_init.sh
		/var/davinci/scripts/minirc_sys_init.sh
		minirc_boot_log "system init finish"
	else
		minirc_boot_log "system init fail"
	fi
}

#start to upgrade firmware 
upgrade_firmware()
{
	if [ -e /var/davinci/scripts/minirc_firmware_upgrade.sh ]; then
		/var/davinci/scripts/minirc_firmware_upgrade.sh |tee -a /var/davinci/log/upgrade.log
	fi
}

sdk_install()
{
	#start to install new version
	if [ -d ${INSTALL_CACHE_DIR} ]; then
		minirc_upgrade_log "start to install package"
		chmod 550 ${INSTALL_CACHE_DIR}/mini_developerkit/scripts/*
		${INSTALL_CACHE_DIR}/mini_developerkit/scripts/minirc_install.sh $INSTALL_CACHE_DIR |tee -a /var/davinci/log/upgrade.log
		#remove rar file
		rm -rf $INSTALL_CACHE_DIR >/dev/null
	fi
}

# ---------------- start ------------
INSTALL_CACHE_DIR="/var/mini/install_cache"

minirc_boot_log "mini boot start"
sdk_install
system_init
upgrade_firmware

exit 0
