#!/bin/bash

minirc_upgrade_log()
{
    cur_time=$(date +%Y-%m-%d,%H:%m:%s)
    echo "$cur_time  $1"
}

INSTALL_CACHE_DIR=$1


files_lib64="libhiai_python_device.so"

files_var="processmonitor matrixdaemon tee.bin aicpufw_monitor hdcd  tee-supplicant 1c3b570d-95ec-4352-bbfe-6d4e58bcd62b.ta
           nve.bin IDE-daemon-device IDE-daemon-host sercure_tool tsch_fw.bin
           dmp_daemon libaicpu.a libcce_aicpudev_alg.a libc_sec.a dt.img sklogd slogd libmedia_server 3559_upgrade_tool minirc_led  minirc_pcb usb_ether.sh"

files_usr_lib64="libdrvaicpu.so libdrvdevdrv.so 
                 libdrvdmp.so libdrvdsmi.so  libdrvhdc.so libdrvhdc_host.so libdrvlog.so
		 libdrvprof.so libdrvtcp.so libdrvupgrade.so libdrvvos.so
                 libDvpp_api.so libDvpp_jpeg_decoder.so libDvpp_jpeg_encoder.so libDvpp_png_decoder.so libDvpp_vpc.so libOMX_hisi_video_decoder.so libOMX_hisi_video_encoder.so
		 libaicpu_engine.so libcce.so libcce_aicore.so libcce_aicpudev.so libcce_aicpudev_alg.a
		 libc_sec.so libssl.so.1.1 libcrypto.so.1.1 libruntime.so libdevmm.so
		 libmodel_encrypt_tool.so libteec.so.1.0 libfmk_common.so libgflags.so.2.2
                 libidedaemon.so libhiai_client.so libhiai_common.so libhiai_server.so libmatrixdaemon.so libproc_bbox.so libmmpa.so libome.so libomg.so
                 libopencv_world.so.3.4 libprofilerclient.so libprofilerserver.so libprotobuf-c.so.1 libprotobuf.so.15
		 libslog.so libmedia_mini.so libfusionengine.so libtiling.so libgraph.so libmemory.so hiaiengine-py2.7.egg hiaiengine-py3.5.egg
         libhiai_python_host2.7.so libhiai_python_host3.5.so"

files_usr_bin="ide_cmd.sh" 
files_usr_local_cce=""

files_etc="hdcBasic.cfg sys_version.conf slog.conf"
files_home="ide_daemon.cfg ide_daemon_cacert.pem ide_daemon_client_cert.pem
           ide_daemon_client_key.pem ide_daemon_server_cert.pem ide_daemon_server_key.pem"
files_firmware="nve.bin xloader.bin lpm3.img HI1910_FPGA_DDR.fd tee.bin Image dt.img filesystem-le.cpio.gz hi3559_ep.tgz upgrade-tool upgrade.cfg opsread-tool mini_upgraded"

username=HwHiAiUser
usergroup=HwHiAiUser
dlog_path=/var/dlog
bbox_path=/var/log/hisi_logs
profiler_path=/usr/local/profiler
matrix_path=/usr/local/matrix
hdcd_path=/var/log/hdcd
ppc_tmp_path=/var/ppc_tmp
python_hiai_path=/usr/local/python-hiai
user_home_path=`eval echo "~${username}"`
ide_daemon_path=`eval echo "~${username}"`/ide_daemon

copy_files()
{
	local files=$1
	local target=$2

	mkdir -p $target
	for file in ${files[@]};
	do
	if [ -e ${INSTALL_CACHE_DIR}/mini_developerkit/$file ]; then
		cp ${INSTALL_CACHE_DIR}/mini_developerkit/$file $target/
		if [ $? -ne 0 ]; then
			minirc_upgrade_log "copy $file to $target fail"
		else
			minirc_upgrade_log "copy $file to $target succ"
		fi
	fi
	done
}

backup_old_version()
{
    if [ -e /var/davinci/sys_version.conf ]; then
		mkdir -p /var/old_version_backup
		rm -rf /var/old_version_backup/*
		cp -r /var/davinci/driver /var/old_version_backup/
		cp -r /var/davinci/firmware /var/old_version_backup/
		cp -r /var/davinci/log /var/old_version_backup/
		cp /var/davinci/sys_version.conf /var/old_version_backup/
	fi
}

subdir_create() {
    path=$1
    if [ ! -d $path ];then
        mkdir -p $path
    fi

    chmod 750 $path
    chown $username:$usergroup $path >> /dev/null 2>&1
    return 0
}

changeMode(){
    chown -f ${username}:${usergroup} $user_home_path
    chown -f ${username}:${usergroup} ${user_home_path}/.bashrc
    chown -f ${username}:${usergroup} ${user_home_path}/.bash_logout
    chown -f ${username}:${usergroup} ${user_home_path}/.profile
    chown -Rf ${username}:${usergroup} $bbox_path
    chown -Rf ${username}:${usergroup} $dlog_path
    find $bbox_path -name history.log | xargs chmod u+w >> /dev/null 2>&1
    chown -f ${username}:${usergroup} /opt/mini/
    chmod -f 640 /etc/hdcBasic.cfg /etc/sys_version.conf /etc/slog.conf $ide_daemon_path/ide_daemon.cfg
    chown -f ${username}:${usergroup} /etc/hdcBasic.cfg /etc/sys_version.conf /etc/slog.conf $ide_daemon_path/ide_daemon.cfg
    
    # ide进程相关文件权限设置
    local chmod_ide_files="ide_daemon_cacert.pem ide_daemon_client_cert.pem ide_daemon_client_key.pem ide_daemon_server_cert.pem ide_daemon_server_key.pem"
    for item in $chmod_ide_files
    do
        chmod -f 600 ${ide_daemon_path}/${item}
        chown -f ${username}:${usergroup} ${ide_daemon_path}/${item}
    done

    if [ -f /usr/bin/ide_cmd.sh ];then
        chattr -i /usr/bin/ide_cmd.sh
    fi

    copy_files "${files_usr_bin[@]}" /usr/bin

    # /usr/bin目录下某些文件权限设置
    local chmod_usr_bin="ide_cmd.sh"
    for item in $chmod_usr_bin
    do
        chmod -f 550 /usr/bin/${item}
        chown -f ${username}:${usergroup} /usr/bin/${item}
    done
    chattr +i /usr/bin/ide_cmd.sh

    # /usr/lib64目录下so文件权限设置
    for item in $files_usr_lib64
    do
        chmod -f 550 /usr/lib64/${item}
        chown -f ${username}:${usergroup} /usr/lib64/${item}
    done

    # /var目录下某些文件权限设置
    local chmod_var="aicpu_fw.bin matrixdaemon tee.bin aicpufw_monitor aicpu_print hdcd tee-supplicant 1c3b570d-95ec-4352-bbfe-6d4e58bcd62b.ta hdctool nve.bin hi1910-emu-4core.dtb
                     tsch_fw hi1910-fpga-2aicore.dtb IDE-daemon-device IDE-daemon-host tsch_fw.bin dmp_daemon libaicpu.a libcce_aicpudev_alg.a
                     libc_sec.a dt.img sklogd slogd libmedia_server 3559_upgrade_tool minirc_led minirc_pcb usb_ether.sh minirc_boot.sh processmonitor"
    for item in $chmod_var
    do
        chmod -f 550 /var/${item}
        chown -f ${username}:${usergroup} /var/${item}
    done

    # /var/davinci/firmware目录相关权限设置
    chmod -f 750 /var/davinci/firmware
    local var_davinci_firmware="nve.bin xloader.bin lpm3.img HI1910_FPGA_DDR.fd tee.bin Image dt.img filesystem-le.cpio.gz hi3559_ep.tgz upgrade-tool upgrade.cfg"
    for item in $var_davinci_firmware
    do
        chmod -f 550 /var/davinci/firmware/${item}
    done

    chmod -f 550 /var/sercure_tool
    chmod -f 750 /var/davinci
    chmod -f 440 /var/davinci/sys_version.conf

    chmod -f 750 /var/davinci/driver
    chmod -f 640 /var/davinci/driver/led
    chmod -f 550 /var/davinci/driver/*.ko

    chmod -f 750 /var/davinci/log
    chmod -f 640 /var/davinci/log/boot_fail_count /var/davinci/log/upgrade.log

    chmod -Rf 550 /var/davinci/scripts

    chmod -f 750 /fw
    chmod -f 550 /fw/*

    chmod -f 750 /var/old_version_backup

    chmod -f 550 /lib64/libcce_devmem.so
    chown -f ${username}:${usergroup}  /lib64/libcce_devmem.so
}

set_security_configuration() {
    local set_write_flag="false"
    if [ ! -w "/etc/sudoers" ];then
        chmod u+w /etc/sudoers
        set_write_flag="true"
    fi

    grep -q "HwHiAiUser" /etc/sudoers
    if [ $? -eq 0 ];then
        sed -i '/HwHiAiUser/c\HwHiAiUser ALL=(root) NOPASSWD:/opt/mini/minirc_install_phase1.sh,/bin/date -s *,/usr/bin/ide_cmd.sh *,/var/sercure_tool *,/bin/sed -i * /etc/network/interfaces,/usr/bin/perf stat *,/usr/bin/perf record *,/usr/bin/perf script *,/usr/bin/pkill perf' /etc/sudoers
    else
        echo "HwHiAiUser ALL=(root) NOPASSWD:/opt/mini/minirc_install_phase1.sh,/bin/date -s *,/usr/bin/ide_cmd.sh *,/var/sercure_tool *,/bin/sed -i * /etc/network/interfaces,/usr/bin/perf stat *,/usr/bin/perf record *,/usr/bin/perf script *,/usr/bin/pkill perf" >> /etc/sudoers
    fi

    if [ $set_write_flag = "true" ];then
        chmod u-w /etc/sudoers
    fi

    unset set_write_flag
    
    return 0
}

install_python_egg(){
    targetdir=$1

    if [ ! -d $targetdir ];then
        echo "$targetdir no exist"
        exit 1
    fi

    # clean environment
    rm -rf ~/.local/bin/easy_install*

    # check setuptools has installed
    PYTHON2_VERSION=`python2 --version 2>&1 | awk 'NR==1{ print $2 }'`
    PYTHON2_VERSION_TAG=`echo ${PYTHON2_VERSION} | awk -F'.' '{ print $1"."$2 }'`
    PYTHON2=python${PYTHON2_VERSION_TAG}
    EASY_INSTALL2_ITEMS="easy_install
                         easy_install-2.1
                         easy_install-2.2
                         easy_install-2.3
                         easy_install-2.4
                         easy_install-2.5
                         easy_install-2.6
                         easy_install-2.7"
    PIP2=pip2

    for item in $(echo $EASY_INSTALL2_ITEMS);
    do
        which ${item} > /dev/null 2>&1
        if [ $? = 0 ];then
           EASY_INSTALL2=$item
           break
        fi
    done

    flag2=true
    which ${EASY_INSTALL2} > /dev/null 2>&1
    if [ $? != 0 ]; then
      flag2=false
    fi
    which ${PIP2} > /dev/null 2>&1
    if [ $? != 0 ]; then
      flag2=false
    fi

    PYTHON3_VERSION=`python3 --version 2>&1 | awk 'NR==1{ print $2 }'`
    PYTHON3_VERSION_TAG=`echo ${PYTHON3_VERSION} | awk -F'.' '{ print $1"."$2 }'`
    PYTHON3=python${PYTHON3_VERSION_TAG}
    EASY_INSTALL3_ITEMS="easy_install3
                         easy_install-3.1
                         easy_install-3.2
                         easy_install-3.3
                         easy_install-3.4
                         easy_install-3.5
                         easy_install-3.6"
    PIP3=pip3

    for item in $(echo $EASY_INSTALL3_ITEMS);
    do
        which ${item} > /dev/null 2>&1
           if [ $? = 0 ];then
              EASY_INSTALL3=$item
              break
           fi
    done

    flag3=true
    which ${EASY_INSTALL3} > /dev/null 2>&1
    if [ $? != 0 ]; then
      flag3=false
    fi
    which ${PIP3} > /dev/null 2>&1
    if [ $? != 0 ]; then
      flag3=false
    fi

    config_file='/usr/local/python-hiai/install_path.config'
    touch $config_file
    # deal with easy_install and setuptools
    if [ "$flag2" = true ];then
        easy_install_version=$($EASY_INSTALL2 --version | awk '{print $2}')
        setuptools_version=$($PIP2 show setuptools | grep -w "Version" | awk '{print $2}')
        if [ -n "${easy_install_version}" ] && [ -n "${setuptools_version}" ];then
            if [ "${easy_install_version}" != "${setuptools_version}" ];then
                ${PIP2} uninstall -q -y setuptools
            fi
        fi

        mkdir -p ${targetdir}/site-packages

        # set PYTHONPATH
        export PYTHONPATH=${targetdir}/site-packages

        # install hiaiengine-python2
        ${EASY_INSTALL2} -q --allow-hosts=None --no-find-links -d ${targetdir}/site-packages ${targetdir}/../../lib64/hiaiengine-py2.7.egg

        # copy common
        which python2 > /dev/null
        if [ $? -eq 0 ];then
            if [ -f ${targetdir}/site-packages/hiaiengine-py2.7.egg/common/get_install_path.py ];then
                ret=$(python2 ${targetdir}/site-packages/hiaiengine-py2.7.egg/common/get_install_path.py)
                if [ ! -n $ret ];then
                    echo "get python install path failed"
                else
                    echo "python2="$ret >$config_file
                    if [ -d $ret/hiai ];then
                        rm -rf $ret/hiai
                        if [ $? != 0 ];then
                            echo "remove hiai failed"
                        fi
                    fi
                    mkdir -p $ret/hiai
                    cp -r ${targetdir}/site-packages/hiaiengine-py2.7.egg/common/* $ret/hiai/
                    if [ $? != 0 ];then
                        echo "copy hiai failed"
                    fi
               fi
            fi
        fi
    fi

    # deal with easy_install and setuptools
    if [ "$flag3" = true ];then
        easy_install_version=$($EASY_INSTALL3 --version | awk '{print $2}')
        setuptools_version=$($PIP3 show setuptools | grep -w "Version" | awk '{print $2}')
        if [ -n "${easy_install_version}" ] && [ -n "${setuptools_version}" ];then
            if [ "${easy_install_version}" != "${setuptools_version}" ];then
                ${PIP3} uninstall -q -y setuptools
            fi
        fi

        mkdir -p ${targetdir}/${PYTHON3}/site-packages

        # set PYTHONPATH
        export PYTHONPATH=${targetdir}/${PYTHON3}/site-packages

        # install hiaiengine-python3
        ${EASY_INSTALL3} -q --allow-hosts=None --no-find-links -d ${targetdir}/${PYTHON3}/site-packages ${targetdir}/../../lib64/hiaiengine-py3.5.egg

        # copy common
        which python3 > /dev/null
        if [ $? -eq 0 ];then
            if [ -f ${targetdir}/${PYTHON3}/site-packages/hiaiengine-py3.5.egg/common/get_install_path.py ];then
                ret=$(python3 ${targetdir}/${PYTHON3}/site-packages/hiaiengine-py3.5.egg/common/get_install_path.py)
                if [ ! -n $ret ];then
                    echo "get python install path failed"
                else
                    echo "python3="$ret >> $config_file
                    if [ -d $ret/hiai ];then
                        rm -rf $ret/hiai
                        if [ $? != 0 ];then
                            echo "remove hiai failed"
                        fi
                    fi
                    mkdir -p $ret/hiai
                    cp -r ${targetdir}/${PYTHON3}/site-packages/hiaiengine-py3.5.egg/common/* $ret/hiai/
                    if [ $? != 0 ];then
                        echo "copy hiai failed"
                    fi
                fi
            fi
        fi

    fi
    touch /etc/hiai_python.config
    echo "/usr" > /etc/hiai_python.config
    echo "DEVICE" >> /etc/hiai_python.config

}


# ---------------- copy start ------------
backup_old_version
minirc_upgrade_log "copy files to target form cache"

grep -qs '/var/dlog' /proc/mounts >> /dev/null 2>&1
ret=$?
if [ -e /dev/mmcblk1p2 ] && [ $ret -ne 0 ]; then
    mkdir /var/oldlog && [ ! -d /var/dlog ] && mkdir /var/dlog && chmod -f 750 /var/dlog && chown -f ${username}:${usergroup} /var/dlog
    rsync -a --include '*/' --exclude '*'  /var/dlog/ /var/oldlog/ >& /dev/null
    mount /dev/mmcblk1p2    /var/dlog
    rsync -a --include '*/' --exclude '*'  /var/oldlog/ /var/dlog/ >& /dev/null
    rm -rf /var/oldlog
fi

grep -qs '/home' /proc/mounts >> /dev/null 2>&1
ret=$?
if [ -e /dev/mmcblk1p3 ] && [ $ret -ne 0 ]; then
    mount /dev/mmcblk1p3    /home
fi

if [ -d /var/davinci ]; then
	rm -rf /var/davinci
fi
if [ -f /var/dlogd ]; then
    rm -f /var/dlogd
fi
if [ -f /etc/dlog.conf ]; then
    rm -f /etc/dlog.conf
fi
if [ ! -d $ide_daemon_path ];then
    mkdir -p $ide_daemon_path
fi

chmod 700 $ide_daemon_path
chown -f $username:$usergroup $ide_daemon_path

copy_files "${files_lib64[@]}"         /lib64
copy_files "${files_var[@]}"           /var
copy_files "${files_usr_lib64[@]}"     /usr/lib64
copy_files "${files_usr_local_cce[@]}" /usr/local/cce/bin
copy_files "${files_etc[@]}"           /etc
copy_files "${files_firmware[@]}"      /var/davinci/firmware
copy_files "${files_home[@]}"          $ide_daemon_path

subdir_create $dlog_path
subdir_create $bbox_path
subdir_create $profiler_path
subdir_create $matrix_path
subdir_create $hdcd_path
subdir_create $ppc_tmp_path
subdir_create $python_hiai_path

mkdir -p /var/davinci/scripts
mkdir -p /var/davinci/driver
mkdir -p /var/davinci/log
[ -f /usr/bin/perf ] && rm /usr/bin/perf
cp ${INSTALL_CACHE_DIR}/mini_developerkit/extend_rootfs/perf_rc /usr/bin/perf
cp ${INSTALL_CACHE_DIR}/mini_developerkit/scripts/*        /var/davinci/scripts/
cp ${INSTALL_CACHE_DIR}/mini_developerkit/sys_version.conf /var/davinci/
cp ${INSTALL_CACHE_DIR}/mini_developerkit/*.ko             /var/davinci/driver/

if [ -e ${INSTALL_CACHE_DIR}/mini_developerkit/firmware_upgrade_progress.log ]; then
cp ${INSTALL_CACHE_DIR}/mini_developerkit/firmware_upgrade_progress.log  /var/davinci/log/
fi

ln -sf /usr/lib64/libopencv_world.so.3.4 /usr/lib64/libopencv_world.so
set_security_configuration
changeMode
install_python_egg $python_hiai_path

if [ -f ${INSTALL_CACHE_DIR}/mini_developerkit/scripts/minirc_install_customized.sh ]; then
	chmod +x ${INSTALL_CACHE_DIR}/mini_developerkit/scripts/minirc_install_customized.sh
	${INSTALL_CACHE_DIR}/mini_developerkit/scripts/minirc_install_customized.sh ${INSTALL_CACHE_DIR}
	if [ $? -ne 0 ]; then
		minirc_upgrade_log "install customized failed"
	fi
fi