#!/bin/bash

LOG_DIR="/var/davinci/log"

minirc_upgrade_log()
{
    cur_time=$(date +%Y-%m-%d,%H:%m:%s)
    echo "$cur_time  $1"
}
username=HwHiAiUser
usergroup=HwHiAiUser

set_led_state()
{
	#stats  booting  upgrading finish
	state=$1
	if [ $state == "booting" ]; then
		echo "led3:1led4:0" > /var/davinci/driver/led
	elif [ $state == "upgrading" ]; then
		echo  "led3:2led4:2" > /var/davinci/driver/led
	elif [ $state == "finish" ]; then
		echo "led3:1led4:1" > /var/davinci/driver/led
	fi
}

minirc_upgrade_progress()
{
	fw=$1
	status=$2
	percent=$3
	
	if [ $status -eq 0 ]; then
		result="succ"
	else
		result="fail"
	fi
	
	echo "$fw upgrade $result ----------------- $percent%" >> ${LOG_DIR}/firmware_upgrade_progress.log
	echo "$fw upgrade result:$status"
}



firmware_upgrade()
{
	sleep 1s
	cd /var/davinci/firmware

	minirc_upgrade_log "!!!!!! start to upgrade firmware, don't power off or reset the board !!!!!!"

	#show version before upgrade
	./upgrade-tool --device_index -1 --component -1 --version

	./upgrade-tool --device_index -1 --component nve --path ./nve.bin
	minirc_upgrade_progress "nve" $? 5

	./upgrade-tool --device_index -1 --component xloader --path ./xloader.bin
	minirc_upgrade_progress "xloader" $? 10

	./upgrade-tool --device_index -1 --component uefi --path ./HI1910_FPGA_DDR.fd
	minirc_upgrade_progress "uefi" $? 15

	#show version after upgrade
	./upgrade-tool --device_index -1 --component -1 --version

	#mark firmware upgraded
	echo "$(date +%Y-%m-%d,%H:%m:%s)  firmware upgraded" > /var/davinci/firmware/mini_upgraded
}

upgrade_3559()
{
	cd /var/

	./3559_upgrade_tool
	
	#mark firmware upgraded
	echo "$(date +%Y-%m-%d,%H:%m:%s)  3559 upgraded" > /var/davinci/firmware/3559_upgraded
}

if [ ! -e /var/davinci/firmware/mini_upgraded ]; then
	minirc_upgrade_log "firmware upgrading ..."

	if [ -e ${LOG_DIR}/firmware_upgrade_progress.log ]; then
		rm ${LOG_DIR}/firmware_upgrade_progress.log
	fi
	echo "firmware start to upgrade  ----------------- 0%" > ${LOG_DIR}/firmware_upgrade_progress.log

	set_led_state "upgrading"
	firmware_upgrade

	if [ -e /dev/pcie_dma ]; then
		upgrade_3559
	fi

	#reboot after upgrade
	minirc_upgrade_log "firmware upgrade finish, start to reboot"
	reboot
else
	ifup eth0
fi

chown -f $username:$usergroup /var/davinci
chown -Rf $username:$usergroup /var/davinci/log
set_led_state "finish"

