#!/bin/bash

# the definition of return value
#1: lack enough sd space
#2: unpack rar package failed
#3: version verify failed
#4: undefined error

minirc_upgrade_log()
{
    if [ $RUN_TYPE == "pc" ]; then
		echo $1
	    return
	fi

    cur_time=$(date +%Y-%m-%d,%H:%m:%s)
	if [ ! -e /var/davinci/log/upgrade.log ]; then
		mkdir -p /var/davinci/log
		echo "$cur_time  create log file" > /var/davinci/log/upgrade.log
	fi
    echo "$cur_time  $1" |tee -a /var/davinci/log/upgrade.log
}


check_space()
{
    if [ $RUN_TYPE == "pc" ]; then
	    return
	fi

	free_size_str=$(df |grep "/dev/root" |awk '{ print $4}')
	if [ "$free_size_str" == "" ]; then
		return 
	fi
	free_size=$((free_size_str))
	if [ $free_size -lt 1048576 ]; then
		minirc_upgrade_log "not enough space left in sdcard, installation failed"
		exit 1
	fi
}

set_led_state()
{
	#stats  booting  upgrading finish
	state=$1
	if [ $state == "booting" ]; then
		echo "led3:1led4:0" > /var/davinci/driver/led
	elif [ $state == "upgrading" ]; then
		echo  "led3:2led4:2" > /var/davinci/driver/led
	elif [ $state == "finish" ]; then
		echo "led3:1led4:1" > /var/davinci/driver/led
	fi
}

minirc_upgrade_progress()
{
	fw=$1
	status=$2
	percent=$3

	if [ $status -eq 0 ]; then
		result="succ"
	else
		result="fail"
		upgrade_result=$status
	fi

	echo "$fw upgrade $result ----------------- $percent%" >> $INSTALL_CACHE_DIR/mini_developerkit/firmware_upgrade_progress.log
	echo "$fw upgrade result:$status"
}

#extrack new version package
extrack_package()
{
	if [ -d ${INSTALL_CACHE_DIR} ]; then
		rm -rf ${INSTALL_CACHE_DIR}
	fi

	minirc_upgrade_log "extrack package, file name:${RAR_FILE}"
	mkdir -p ${INSTALL_CACHE_DIR}
	unzip ${RAR_FILE} -d ${INSTALL_CACHE_DIR}
	if [ $? -ne 0 ]; then
		minirc_upgrade_log "extrack package(${RAR_FILE}) fail, installation failed"
		which unzip
		if [ $? -ne 0 ]; then
			minirc_upgrade_log "can't find unzip tool, please install unzip tool first"
		else
			minirc_upgrade_log "maybe the package is corrupted, please check it"
		fi
		rm -rf ${INSTALL_CACHE_DIR}
		exit 2
	else
		minirc_upgrade_log "extrack package succ"
    fi
}


install_phase1()
{
	chmod +x $INSTALL_CACHE_DIR/mini_developerkit/scripts/*
	$INSTALL_CACHE_DIR/mini_developerkit/scripts/minirc_install_pre.sh $SDCARD_MOUNT_DIR
	result=$?
	if [ $result -ne 0 ]; then
		rm -rf ${INSTALL_CACHE_DIR}
		exit $result
	fi
}

firmware_upgrade()
{
	sleep 1s
	if [ ! -e /var/mini/install_cache/mini_developerkit ]; then
		return 0
	fi

	minirc_upgrade_log "!!!!!! start to upgrade firmware, don't power off or reset the board !!!!!!"

	#show version before upgrade
	$FIRMWARE_PATH/upgrade-tool --device_index -1 --component -1 --version

	$INSTALL_CACHE_DIR/mini_developerkit/upgrade-tool --device_index -1 --component m3fw --path $INSTALL_CACHE_DIR/mini_developerkit/lpm3.img
	minirc_upgrade_progress "m3fw" $? 5

	$INSTALL_CACHE_DIR/mini_developerkit/upgrade-tool --device_index -1 --component kernel --path $INSTALL_CACHE_DIR/mini_developerkit/Image
	minirc_upgrade_progress "kernel" $? 15

	$INSTALL_CACHE_DIR/mini_developerkit/upgrade-tool --device_index -1 --component tee --path $INSTALL_CACHE_DIR/mini_developerkit/tee.bin
	minirc_upgrade_progress "tee" $? 30

	$INSTALL_CACHE_DIR/mini_developerkit/upgrade-tool --device_index -1 --component dtb --path $INSTALL_CACHE_DIR/mini_developerkit/dt.img
	minirc_upgrade_progress "dtb" $? 50
	
	$INSTALL_CACHE_DIR/mini_developerkit/upgrade-tool --device_index -1 --component nve --path $INSTALL_CACHE_DIR/mini_developerkit/nve.bin
	minirc_upgrade_progress "nve" $? 75

	$INSTALL_CACHE_DIR/mini_developerkit/upgrade-tool --device_index -1 --component xloader --path $INSTALL_CACHE_DIR/mini_developerkit/xloader.bin
	minirc_upgrade_progress "xloader" $? 85

	$INSTALL_CACHE_DIR/mini_developerkit/upgrade-tool --device_index -1 --component uefi --path $INSTALL_CACHE_DIR/mini_developerkit/HI1910_FPGA_DDR.fd
	minirc_upgrade_progress "uefi" $? 100

	#mark firmware upgraded
	if [ $upgrade_result -eq 0 ]; then
		echo "$(date +%Y-%m-%d,%H:%m:%s)  firmware upgraded" > $INSTALL_CACHE_DIR/mini_developerkit/mini_upgraded
		if [ -e $FIRMWARE_PATH/upgrade_failed ]; then
			rm $FIRMWARE_PATH/upgrade_failed
		fi
	fi
}

upgrade_3559()
{
	lspci | grep 3559
	if [ $? -ne 0 ]; then
		return 0
	fi

	cd /var/
	./3559_upgrade_tool
	#mark firmware upgraded
	echo "$(date +%Y-%m-%d,%H:%M:%S)  3559 upgraded" > /var/davinci/firmware/3559_upgraded
}


INSTALL_CACHE_DIR="/var/mini/install_cache"
FIRMWARE_PATH="/var/davinci/firmware"
RAR_DIR="/opt/mini"
upgrade_result=0

if [ -e /var/minirc_led ];then
	set_led_state "upgrading"
	/var/minirc_led &
fi
if [ -n "$1" ];then
	SDCARD_MOUNT_DIR=$1
	INSTALL_CACHE_DIR="$SDCARD_MOUNT_DIR/var/mini/install_cache"
	RAR_DIR="$SDCARD_MOUNT_DIR/opt/mini"
	RUN_TYPE="pc"
else
	SDCARD_MOUNT_DIR=""
	INSTALL_CACHE_DIR="/var/mini/install_cache"
	RAR_DIR="/opt/mini"
	RUN_TYPE="mini"
fi

echo "RAR_DIR=$RAR_DIR"
echo "INSTALL_CACHE_DIR=$INSTALL_CACHE_DIR"
echo "RUN_TYPE=$RUN_TYPE"

if [ ! -e $RAR_DIR/mini_developerkit*.rar ]; then
	minirc_upgrade_log "can't not find $RAR_DIR/mini_developerkit*.rar, installation failed"
	exit 4
fi

RAR_FILE_NUM=$(ls $RAR_DIR | grep ".rar" | grep -c "mini_developerkit")
if [ $RAR_FILE_NUM -ge 1 ];then
	RAR_FILE=$(ls $RAR_DIR | grep ".rar" | grep "mini_developerkit" | sort -rV | head -n 1)
	RAR_FILE=$RAR_DIR/$RAR_FILE

	check_space
	if [ -d $FIRMWARE_PATH ]; then
		echo "$(date +%Y-%m-%d,%H:%m:%s) firmware upgrade failed" > $FIRMWARE_PATH/upgrade_failed
	fi
	extrack_package
	install_phase1
        firmware_upgrade
	upgrade_3559
	set_led_state "finish"

	rm $RAR_DIR/mini_developerkit*.rar
fi

sync

if [ $RUN_TYPE == "mini" ]; then
	minirc_upgrade_log "the first phase of installation finish, please reboot"
fi

exit 0
